@@ -5,7 +5,7 @@ Wrapper for YAJL C library version 1.x.
 from ctypes import Structure, c_uint, c_ubyte, c_int, c_long, c_double, c_char, \
                    c_void_p, c_char_p, CFUNCTYPE, POINTER, byref, string_at, cast
 
-from ijson import common, backends
+from ijson import common, backends, utils
 from ijson.compat import b2s
 
 
@@ -29,12 +29,12 @@ _callback_data = [
     ('boolean', C_INT, lambda v: bool(v)),
     # "integer" and "double" aren't actually yielded by yajl since "number"
     # takes precedence if defined
-    ('integer', C_LONG, lambda v, l: int(string_at(v, l))),
-    ('double', C_DOUBLE, lambda v, l: float(string_at(v, l))),
+    ('integer', C_LONG, lambda *_args: None),
+    ('double', C_DOUBLE, lambda *_args: None),
     ('number', C_STR, lambda v, l: common.number(b2s(string_at(v, l)))),
     ('string', C_STR, lambda v, l: string_at(v, l).decode('utf-8')),
     ('start_map', C_EMPTY, lambda: None),
-    ('map_key', C_STR, lambda v, l: b2s(string_at(v, l))),
+    ('map_key', C_STR, lambda v, l: string_at(v, l).decode('utf-8')),
     ('end_map', C_EMPTY, lambda: None),
     ('start_array', C_EMPTY, lambda: None),
     ('end_array', C_EMPTY, lambda: None),
@@ -55,7 +55,8 @@ YAJL_INSUFFICIENT_DATA = 2
 YAJL_ERROR = 3
 
 
-def basic_parse(f, allow_comments=False, check_utf8=False, buf_size=64 * 1024):
+@utils.coroutine
+def basic_parse_basecoro(target, allow_comments=False, check_utf8=False):
     '''
     Iterator yielding unprefixed events.
 
@@ -66,11 +67,11 @@ def basic_parse(f, allow_comments=False, check_utf8=False, buf_size=64 * 1024):
     - check_utf8: if True, parser will cause an error if input is invalid utf-8
     - buf_size: a size of an input buffer
     '''
-    events = []
+    send = target.send
 
     def callback(event, func_type, func):
         def c_callback(context, *args):
-            events.append((event, func(*args)))
+            send((event, func(*args)))
             return 1
         return func_type(c_callback)
 
@@ -79,7 +80,10 @@ def basic_parse(f, allow_comments=False, check_utf8=False, buf_size=64 * 1024):
     handle = yajl.yajl_alloc(byref(callbacks), byref(config), None, None)
     try:
         while True:
-            buffer = f.read(buf_size)
+            try:
+                buffer = (yield)
+            except GeneratorExit:
+                buffer = b''
             if buffer:
                 result = yajl.yajl_parse(handle, buffer, len(buffer))
             else:
@@ -88,27 +92,13 @@ def basic_parse(f, allow_comments=False, check_utf8=False, buf_size=64 * 1024):
                 perror = yajl.yajl_get_error(handle, 1, buffer, len(buffer))
                 error = cast(perror, c_char_p).value
                 yajl.yajl_free_error(handle, perror)
-                exception = common.IncompleteJSONError if result == YAJL_INSUFFICIENT_DATA else common.JSONError
-                raise common.JSONError(error)
-            if not buffer and not events:
+                raise common.JSONError(error.decode('utf-8'))
+            elif not buffer:
                 if result == YAJL_INSUFFICIENT_DATA:
                     raise common.IncompleteJSONError('Incomplete JSON data')
                 break
-
-            for event in events:
-                yield event
-            events = []
     finally:
         yajl.yajl_free(handle)
 
-def parse(file, **kwargs):
-    '''
-    Backend-specific wrapper for ijson.common.parse.
-    '''
-    return common.parse(basic_parse(file, **kwargs))
 
-def items(file, prefix):
-    '''
-    Backend-specific wrapper for ijson.common.items.
-    '''
-    return common.items(parse(file), prefix)
+common.enrich_backend(globals())
